@@ -10,7 +10,7 @@ standardize.estimate_predicted <- function(x, include_response = TRUE, ...) {
   x[names(data)] <- datawizard::standardize(as.data.frame(x)[names(data)], reference = data, ...)
 
   # Standardize response
-  if (include_response == TRUE && insight::model_info(attributes(x)$model)$is_linear) {
+  if (include_response && insight::model_info(attributes(x)$model)$is_linear) {
     resp <- insight::get_response(attributes(x)$model)
     disp <- attributes(datawizard::standardize(resp, ...))$scale
 
@@ -69,16 +69,18 @@ standardize.estimate_slopes <- standardize.estimate_contrasts
 #' @method unstandardize estimate_predicted
 #' @export
 unstandardize.estimate_predicted <- function(x, include_response = TRUE, ...) {
+  model <- attributes(x)$model
+
   # Get data of predictors
-  data <- insight::get_data(attributes(x)$model, verbose = FALSE, ...)
+  data <- insight::get_data(model, verbose = FALSE, ...)
   data[[attributes(x)$response]] <- NULL # Remove resp from data
 
   # Standardize predictors
   x[names(data)] <- datawizard::unstandardize(as.data.frame(x)[names(data)], reference = data, ...)
 
   # Standardize response
-  if (include_response == TRUE && insight::model_info(attributes(x)$model)$is_linear) {
-    resp <- insight::get_response(attributes(x)$model)
+  if (include_response == TRUE && insight::model_info(model)$is_linear) {
+    resp <- insight::get_response(model)
     disp <- attributes(datawizard::standardize(resp, ...))$scale
 
     for (col in c("Predicted", "Mean", "CI_low", "CI_high")) {
@@ -95,3 +97,31 @@ unstandardize.estimate_predicted <- function(x, include_response = TRUE, ...) {
   }
   x
 }
+
+
+#' @export
+unstandardize.estimate_means <- unstandardize.estimate_predicted
+
+
+#' @export
+unstandardize.estimate_contrasts <- function(x, robust = FALSE, ...) {
+  model <- attributes(x)$model
+
+  if (insight::model_info(model)$is_linear) {
+    # Get dispersion scaling factor
+    if (robust) {
+      disp <- stats::mad(insight::get_response(model), na.rm = TRUE)
+    } else {
+      disp <- stats::sd(insight::get_response(model), na.rm = TRUE)
+    }
+
+    # Standardize relevant cols
+    for (col in c("Difference", "Ratio", "Coefficient", "SE", "MAD", "CI_low", "CI_high")) {
+      if (col %in% names(x)) {
+        x[col] <- x[[col]] * disp
+      }
+    }
+  }
+
+  x
+}
